/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DFK31AU03STATE_H
#define DFK31AU03STATE_H

#include <Driver.h>
#include "Dfk31Au03Driver.h"

namespace dfk41au03 {

/*!
 * \class Dfk31Au03State
 * \brief Virtual class for implement State pattern
 *
 * Define all methods that perfect, simulated and real classes have to implement
 */
class Dfk31Au03State: public DriverState<Dfk31Au03Driver> {

public:

	/*!
	 * \brief Constructor
	 * \param[in] owner The device driver main class link
	 */
	Dfk31Au03State(Dfk31Au03Driver* owner) :
			DriverState<Dfk31Au03Driver>(owner) {
	}
	/*!
	 * \brief Destructor
	 */
	virtual ~Dfk31Au03State() {
	}

	/*!
	 * \brief Clear command implementation
	 */
	virtual void clear() = 0;

	/*!
	 * \brief Write Param command implementation
	 */
	virtual void writeParam() = 0;

	/*!
	 * \brief Read command implementation
	 */
	virtual void read() = 0;

	/*!
	 * \brief Synchronize Read command implementation
	 */
	virtual void synchroniseRead() = 0;

	/*!
	 * \brief Start command implementation
	 */
	virtual void start() = 0;

	/*!
	 * \brief Resume command implementation
	 */
	virtual void resume() = 0;

	/*!
	 * \brief Pause command implementation
	 */
	virtual void pause() = 0;

	/*!
	 * \brief Stop command implementation
	 */
	virtual void stop() = 0;

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readStatus() = 0;

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readInfos() = 0;

};

}
#endif //DFK31AU03STATE_H
