/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef REALDFK31AU03DRIVER_H
#define REALDFK31AU03DRIVER_H

#include "Dfk31Au03State.h"
#include "Dfk31Au03Def.h"

#include "Utilities/Counter.h"

namespace dfk41au03 {

/*!
 * \class RealDfk31Au03Driver
 * \brief Real implementation class for the Dfk31Au03 device driver
 *
 * This class is a real implementation of Dfk31Au03 device driver.
 */
class RealDfk31Au03Driver: public Dfk31Au03State {

public:

	/*!
	 * \brief Constructor
	 * \param[in] owner The device driver main class link
	 */
	RealDfk31Au03Driver(Dfk31Au03Driver* owner);

	/*!
	 * \brief Destructor
	 */
	virtual ~RealDfk31Au03Driver();

	/*!
	 * \brief Init command implementation
	 */
	virtual void init();
	/*!
	 * \brief Clear command implementation
	 */
	virtual void clear();

	/*!
	 * \brief Write Param command implementation
	 */
	virtual void writeParam();

	/*!
	 * \brief Read command implementation
	 */
	virtual void read();

	/*!
	 * \brief Synchronize Read command implementation
	 */
	virtual void synchroniseRead();

	/*!
	 * \brief Start command implementation
	 */
	virtual void start();

	/*!
	 * \brief Resume command implementation
	 */
	virtual void resume();

	/*!
	 * \brief Pause command implementation
	 */
	virtual void pause();

	/*!
	 * \brief Stop command implementation
	 */
	virtual void stop();

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readStatus();

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readInfos();

private:
//	uint8_t *buffer;
#define CLEAR(x) memset(&(x), 0, sizeof(x))
	uint8 *buffer;
	struct buffer {
		void *start;
		size_t length;
	};
	int16 m_Handle;
	Counter m_TimeCounter;
	uint8 *dest_buf;

	void v4lconvert_yuyv_to_rgb24(const uint8 *src, uint8 *dest, int32 width, int32 height, int32 stride);

	void errno_exit(const char *s);

	int32 xioctl(int32 fd, int32 request, void *arg);

	void process_image(const void *p_buf, const int32 len_buf);

	int32 read_frame(int32 count);

	void mainloop();

	void stop_capturing();

	void start_capturing(void);

	void uninit_device();

	void init_mmap();

	void init_device();

	void close_device();

	void open_device();

	int32 fd;
	struct buffer * buffers;
	uint32 n_buffers;

#define CLIP(color) (uint8)(((color)>0xFF)?0xff:(((color)<0)?0:(color)))

};

}
#endif //REALDFK31AU03DRIVER_H
