/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef PSLDEF_H
#define PSLDEF_H


namespace psl {

static const std::string MILLISEC = "Millisec";
static const std::string SECOND = "Second";

static const std::string STANDARD = "Standard"; 	 // turn the camera Standard'
static const std::string AVERAGE = "Average"; 	 // turn the camera Average'
static const std::string SUMMATION = "Summation"; 	 // turn the camera Summation'
static const std::string ACCUMULATION = "Accumulation"; 	 // turn the camera Accumulation'



static const std::string OPEN = "Open"; 	 // turn the camera on'
static const std::string CLOSE = "Close";	 // turn the camera off
static const std::string RESET= "Reset"; 	 //turn off then turn on the camera
static const std::string SNAP = "Snap"; 	 //Capture an image
static const std::string ABORT_SNAP = "AbortSnap"; //'AbortSnap' => Abort an image acquisition
static const std::string SAVE_IMG = "Save"; 	 //'Save' => save the captured image at:
static const std::string FILE_DIRECTORY = "FileDirectory"; //'FileDirectory'\ 'FilePreffix'+'FileRefNumber'+'.'+'FileFormat'
static const std::string CONNECTED = "CLIENT_CONNECTED"; //'CLIENT_CONNECTED' => return: "!!!Welcome on PSL server!!!" (str)
static const std::string GET_IMAGE = "GetImage"; //'GetImage' => return: ((width,height), data) ((int,int),str)
static const std::string GET_MODE = "GetMode";	 //'GetMode' => return: image data type. Could be {'L', 'I;16', 'I', 'F'}.
static const std::string GET_EXPOSURE = "GetExposure";	 //'GetExposure' => return: Exposure (float) in milliseconds
static const std::string GET_BINNING = "GetBinning";	 //'GetBinning' => return: (xbin,ybin) (int,int)
static const std::string GET_SUB_AREA = "GetSubArea";	 //'GetSubArea' => return: (left,top,right,bottom) (int,int,int,int)
static const std::string GET_TRIGGER_MODE = "GetTriggerModes"; //'GetTriggerModes' => return: TriggerMode (str)
static const std::string GET_FILE_DIRECTORY = "GetFileDirectory";	 //'GetFileDirectory' => return: FileDirectory (str)
static const std::string GET_FILE_PREFIX = "GetFilePreffix"; //'GetFilePreffix' => return: FilePreffix (str)
static const std::string GET_FILE_SUFFIX = "GetFileSuffix"; //'GetFileSuffix' => return: FileSuffix (str)
static const std::string GET_FILE_REF_NUMBER = "GetFileRefNumber";	 //'GetFileRefNumber' => return: FileRefNumber (str)
static const std::string GET_FILE_FORMAT = "GetFileFormat"; //'GetFileFormat' => return: FileFormat (str)
static const std::string SET_EXPOSURE = "SetExposure";	 //'SetExposure;1.33'
static const std::string SET_SHOW_AVERAGE = "SetShowAverage";	 //'SetShowAverage;1.33'
static const std::string SET_AUTO_SAVE = "SetAutoSave";	 //'SetAutoSave;1.33'
static const std::string SET_BRIGHTPIXEL = "SetBrightPixel";	 //'SetBrightPixel'
static const std::string SET_NB_FRAME = "SetFrameNumber";	 //'SetFrameNumber'
static const std::string SET_BINNING = "SetBinning";	 //'SetBinning;2;2'
static const std::string SET_GAIN = "SetIntensifierGain";	 //'SetIntensifierGain'
static const std::string SET_ACQ_MODE = "SetAcquisitionMode";	 //'SetAcquisitionMode'
static const std::string SET_SUB_AREA = "SetSubArea";	 //'SetSubArea;100;100;800;800'
static const std::string SET_TRIGGER_MODE = "SetTriggerModes"; //'SetTriggerModes
static const std::string SOFTWARE = "Software";	 //trigger_modes = Software
static const std::string FREERUN = "FreeRunning"; //trigger_modes = FreeRunning
static const std::string HARDWARE_FALLING = "Hardware_Falling"; //trigger_modes =Hardware_Falling
static const std::string HARDWARE_RISING = "Hardware_Rising"; //trigger_modes =Hardware_Rising
static const std::string PIPLINE_FALLING = "Pipeline_Falling"; //trigger_modes =Pipeline_Falling
static const std::string PIPLINE_RISING = "Pipeline_Rising"; //trigger_modes =Pipeline_Rising
static const std::string PIPLINE_SOFTWARE = "Pipeline_Software";	 //trigger_modes =Pipeline_Software
static const std::string SET_FILE_DIRECTORY = "SetFileDirectory";	 //'SetFileDirectory;.'
static const std::string SET_FILE_PREFIX = "SetFilePreffix"; //'SetFilePreffix;pref'
static const std::string SET_FILE_SUFFIX = "SetFileSuffix"; //'SetFileSuffix;suf'
static const std::string SET_FILE_REF_NUMBER = "SetFileRefNumber";	 //'SetFileRefNumber;1234'
static const std::string GET_FILE_RW_FLAG = "SetFileRWFlag"; //'SetFileRWFlag;True'
static const std::string SET_FILE_FORMAT = "SetFileFormat"; //'SetFileFormat;tiff'
static const std::string SEPARATOR = ";"; //Note: for all “Set*” commands use “;” as separator.
static const std::string WAIT_FOR_IMAGE = "WaitForImage"; //WaitForImage




enum
{
	READOUT_NOT_ACTIVE = 0,
	EXPOSURE_IN_PROGRESS,
	READOUT_IN_PROGRESS,
	READOUT_COMPLETE,
	FRAME_AVAILABLE = READOUT_COMPLETE,
	READOUT_FAILED,
	ACQUISITION_IN_PROGRESS,
	MAX_CAMERA_STATUS
};


enum
{
	MILLISEC_MODE,
	SECOND_MODE,
};





static const std::string RGB = "RGB";
static const std::string PNG = "PNG";
static const int32 RGB24BIT = 24;
static const int32 XSIZE = 1024;
static const int32 YSIZE = 768;
static const int32 COUNT_IGNORE	= 3;	// frame count for initilize camera


#define CAM_NAME_LEN 	32
#define PARAM_SER_SIZE	((2<<16) + (6<<24) + 58)
#define PARAM_PAR_SIZE	((2<<16) + (6<<24) + 57)
#define ERROR_MSG_LEN 255

#define DEFAULT_TIME 	0.1
#define DEFAULT_HEIGHT 	2048
#define DEFAULT_WIDTH 	2048
static const std::string TERMINATOR = "\r";


static const int32 COUNTING = 1;
static const int32 END = 2;


}

#endif //PSLDEF_H
