/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealPslDriver.h"
#include "PslDef.h"
#include "PslDriver.h"
#include <boost/format.hpp>
#include "Utilities/Counter.h"

#include <cstdio>

namespace psl {

/*
 * Constructor
 */
RealPslDriver::RealPslDriver(PslDriver* owner) :
		PslState(owner) {
	/* Empty */
}

/*
 * Destructor
 */
RealPslDriver::~RealPslDriver() {
}

/*
 * init
 */
void RealPslDriver::init() {
	readInfos();
}

/*
 * clear
 */
void RealPslDriver::clear() {
}

/*
 * writeParam
 */
void RealPslDriver::writeParam() {

	ostringstream buf_conf;



	string err;

	buf_conf << "SetAcquisitionMode;"<< boost::format("%03d") %PslDriver::MODE[owner()->acqmode()];
	 err = writeAndRead(buf_conf.str(), TERMINATOR);
	 buf_conf.str("");
	 buf_conf.clear();
	if (err != "True") {
		cerr << "error SetAcquisitionMode " << endl;
	}
	buf_conf << "SetAutoSave;1";
	 err = writeAndRead(buf_conf.str(), TERMINATOR);
	 buf_conf.str("");
	buf_conf.clear();
	if (err != "True") {
		cerr << "error SetAutoSave " << endl;
	}

	buf_conf << "SetShowAverage;0";
	 err = writeAndRead(buf_conf.str(), TERMINATOR);
	 buf_conf.str("");
	buf_conf.clear();
	if (err != "True") {
		cerr << "error SetShowAverage " << endl;
	}

	buf_conf << "SetBrightPixel;" <<  boost::format("%03d") % owner()->bpixel();
	 err = writeAndRead(buf_conf.str(), TERMINATOR);
	 buf_conf.str("");
	buf_conf.clear();
	if (err != "True") {
		cerr << "error SetBrightPixel " << endl;
	}

	buf_conf << "SetIntensifierGain;"<< boost::format("%03d") %  owner()->gain();
	 err = writeAndRead(buf_conf.str(), TERMINATOR);
	 buf_conf.str("");
	buf_conf.clear();
	if (err != "True") {
		cerr << "error SetIntensifierGain " << endl;
	}
	owner()->nbFrame=1;
	buf_conf << "SetFrameNumber;"<<  boost::format("%03d") % owner()->nbFrame();
	 err = writeAndRead(buf_conf.str(), TERMINATOR);
	 buf_conf.str("");
	buf_conf.clear();
	if (err != "True") {
		cerr << "error SetFrameNumber " << endl;
	}

}

/*
 * synchroniseRead
 */
void RealPslDriver::synchroniseRead() {
	string err;
	int32 i;
//	do {
//		err = writeAndRead("WaitForImage", TERMINATOR);
//	} while ((err == "False") && (i++ < 5));
//	if (err != "True") {
//		cerr << "error acquisition " << endl;
//	} else {
		string buf;
		owner()->connect();
		Counter c1;
		cout << "<-- GetImage" << endl;
		owner()->write("GetImage", TERMINATOR);
		buf = owner()->read(16);
	//	owner()->disconnect();
		cout << "--> buf " << buf << endl;
		typedef tokenizer<char_separator<char> > tokenizer;
		char_separator<char> sep(";");
		tokenizer tok1(buf, sep);
		tokenizer::iterator iter = tok1.begin();
		if (iter != tok1.end()) {
			cout << " iter 1 : " << *iter << endl;
			++iter;
			if (iter != tok1.end()) {
				cout << " iter 2 : " << *iter << endl;
				int32 xsize = lexical_cast<int32>(*iter);
				++iter;
				if (iter != tok1.end()) {
					cout << " iter 3 : " << *iter << endl;
					int32 ysize = lexical_cast<int32>(*iter);;
					++iter;
					if (iter != tok1.end()) {
						cout << " iter 4 : " << *iter << endl;
						int32 size = xsize * ysize;//lexical_cast<int32>(*iter);
						//sleep(5);
						string datas = owner()->read(size * 2);
						uint16* tmp = (uint16*) (datas.c_str());
						cout << "datas size = " << datas.size() << endl;
						int32* data = owner()->data();
						int32 dataSize = owner()->dataSize();
						//memset(data, 0, dataSize * sizeof(int32));
						float64 sum = 0;
						if ((data != NULL) && (size == dataSize)) {
							for (int32 x = 0; x < xsize; ++x) {
								for (int32 y = 0; y < ysize; ++y) {
//									data[y + (xsize - x - 1) * ysize] = tmp[x + y * xsize];
									data[y + x * ysize] = tmp[y + x * ysize];
									sum += (float64) data[y + x * ysize];
								}
							}
							owner()->sum = sum;
							owner()->data.sendEvent();
//							for (int32 i = 0; i < dataSize; ++i) {
//								data[i] = (int32) datas[i];
//								sum += (float64) data[i];
//							}
						} else {
							cerr << "error mask " << endl;

						}
					}
				}
			}
		}
		owner()->disconnect();
		cout << "read time:" << c1.getTime() << endl;

	cout << "owner()->m_SynchroniseReadTerminated = true" << endl;
	owner()->m_SynchroniseReadTerminated = true;
}

/*
 * read
 */
void RealPslDriver::read() {
}

/*
 * start
 */
void RealPslDriver::start() {
	ostringstream buf_conf;
	float64 exp_time = owner()->time.setpoint();
	string err;


	 buf_conf << "SetRecordPath;"<< "z:";// owner()->fileDirectory();
	 err = writeAndRead(buf_conf.str(), TERMINATOR);
	 buf_conf.str("");
	 buf_conf.clear();
	if (err != "True") {
		cerr << "error SetRecordPath; " << endl;
	}
	 buf_conf << "SetRecordName;"<< "test";//  owner()->fileName();
	 err = writeAndRead(buf_conf.str(), TERMINATOR);
	 buf_conf.str("");
	 buf_conf.clear();
	if (err != "True") {
		cerr << "error SetRecordName;; " << endl;
	}
	owner()->numor=222;
	 buf_conf << "SetRecordNumber;"<< owner()->numor();
	 err = writeAndRead(buf_conf.str(), TERMINATOR);
	 buf_conf.str("");
	 buf_conf.clear();
	if (err != "True") {
		cerr << "error SetRecordName;; " << endl;
	}

	cout << "start" << endl;
	cout << "exp_time:" << exp_time << endl;
	buf_conf << "SetExposure;[" << exp_time << ",\"Second\"]";

	 err = writeAndRead(buf_conf.str(), TERMINATOR);

	err = writeAndRead("Snap", TERMINATOR);
	if (err != "True") {
		cerr << "error acquisition " << endl;
		owner()->startActivated = false;
		owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	}
	m_TimeCounter.clear();
}

/*
 * resume
 */
void RealPslDriver::resume() {
}

/*
 * pause
 */
void RealPslDriver::pause() {

}

/*
 * stop
 */
void RealPslDriver::stop() {
	ostringstream buf_cmd2;
	writeAndRead("Stop", TERMINATOR);
	owner()->startActivated = false;
	owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
}

/*
 * readStatus
 */
void RealPslDriver::readStatus() {

	float64 actualtime = m_TimeCounter.getTime();
	owner()->time = actualtime;

	int32 status = 0;
	if (owner()->startActivated() == true) {
		if (actualtime > owner()->time.setpoint()) {
			status |= END;
			owner()->startActivated = false;
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		} else {
			status |= COUNTING;
			owner()->calculateProgression();
		}
	} else {
		cout << "m_SynchroniseReadActivated = " << owner()->m_SynchroniseReadActivated << endl;
		cout << "m_SynchroniseReadTerminated = " << owner()->m_SynchroniseReadTerminated << endl;
		if (owner()->m_SynchroniseReadActivated == true) {
			if (owner()->m_SynchroniseReadTerminated == true) {
				owner()->m_SynchroniseReadActivated = false;
				owner()->m_SynchroniseReadTerminated = false;
				owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
			}
		} else {
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		}
	}
	owner()->computeGateStatus(status);
	owner()->computeAcquisitionStatus(0);
	owner()->computeDetectorStatus(0);
}

/*
 * readInfos
 */
void RealPslDriver::readInfos() {
	string info = writeAndRead("GetVersion", TERMINATOR);
	owner()->version = info;
}

/*
 * writeAndRead
 */
std::string RealPslDriver::writeAndRead(const std::string& command,
		const std::string& terminator) {
	string buf;
	owner()->connect();
	cout << "<-- " << command << endl;
	owner()->write(command, terminator);
	buf = owner()->read();
	cout << "--> " << buf << endl;
	owner()->disconnect();
	return buf;

}

}
